import { Server as HTTPServer } from "node:http";
import { Express, Router } from "express";
import "express-async-errors";
declare const InternalVixConfigure: unique symbol;
export declare abstract class VixCore {
    title: string;
    port: number;
    app: Express;
    httpServer: HTTPServer;
    constructor(title: string, port?: number);
    protected preConfigure(): Promise<void>;
    protected [InternalVixConfigure]: () => Promise<void>;
    protected abstract configureServices(): Promise<void>;
    protected postConfigure(): Promise<void>;
    protected abstract startServices(): Promise<void>;
    protected abstract onStart(): Promise<void>;
    start(): Promise<any>;
}
export declare type VixpressServices = Map<string, typeof VixpressService>;
export declare class VixpressService {
    app: Express;
    httpServer: HTTPServer;
    constructor(app: Express, httpServer: HTTPServer);
    configureService(): Promise<void>;
    startService(): Promise<void>;
}
export declare abstract class Vix extends VixCore {
    private serviceClasses;
    services: Map<string, VixpressService>;
    private initialized;
    constructor(title: string, port?: number, services?: VixpressServices);
    protected [InternalVixConfigure]: () => Promise<void>;
    protected configureServices(): Promise<void>;
    setService(serviceKey: string, service: typeof VixpressService): void;
    protected startServices(): Promise<void>;
    protected onStart(): Promise<void>;
}
export declare abstract class Vixpress extends Vix {
    private static RouterService;
    private static Tip;
    private routerService;
    private routerReady;
    constructor(title: string, port?: number);
    protected setRouter(RouterService: typeof VixpressService): void;
    configureServices(): Promise<void>;
    generateApi(): Promise<string>;
    setService(serviceKey: string, Service: typeof VixpressService): void;
}
export declare abstract class VixpressController {
    protected app: Express;
    constructor(app: Express);
}
export declare abstract class VixpressRoute {
    protected app: Express;
    router: Router;
    constructor(app: Express);
    protected buildRouter(): import("express-serve-static-core").Router;
    abstract configureRoutes(router: Router): Promise<void>;
    useController<T extends VixpressController>(Controller: new (app: Express) => T): T;
}
export declare abstract class VixpressRouter extends VixpressService {
    router: Router;
    protected abstract baseUrl: string;
    protected abstract routerImportUrl: string | URL;
    constructor(app: Express, httpServer: HTTPServer);
    protected buildRouter(): import("express-serve-static-core").Router;
    protected abstract configureRoutes(router: Router): Promise<void>;
    protected useRoute(route: string, ControllerRoute: new (app: Express) => VixpressRoute): Promise<void>;
    configureService(): Promise<void>;
    protected configureCore(): void;
    protected configureVite(): void;
    protected configureErrorHandling(): void;
}
export {};
