import ClientError from "../express/ClientErrors.js";
async function coreRequest(apiRequestArgs) {
    const { url, json, body, method, jsonify, extraHeaders, handleErrors } = apiRequestArgs;
    const headers = extraHeaders ?? {};
    const requestMethod = method ?? (!!json ? "POST" : "GET");
    const requestBody = body ?? (!!json ? JSON.stringify(json) : undefined);
    if (!!json)
        headers["Content-Type"] = "application/json";
    const doHandleErrors = handleErrors ?? true;
    const noJsonify = jsonify === false;
    return fetch(url, {
        method: requestMethod,
        body: requestBody,
        headers,
    }).then((res) => {
        if (res.status >= 300 && doHandleErrors) {
            return res
                .json()
                .then((responseError) => {
                const { msg, code } = responseError;
                throw new ClientError(code, msg);
            })
                .catch((e) => {
                if (e instanceof ClientError)
                    throw e;
                throw new ClientError(res.status, res.statusText);
            });
        }
        if (noJsonify)
            return res;
        return res.json().catch(() => undefined);
    });
}
export async function rawRequest(apiRequestArgs, getAuthHeader) {
    if (!getAuthHeader)
        return coreRequest(apiRequestArgs);
    const extraHeaders = apiRequestArgs.extraHeaders ?? {};
    const authHeaders = getAuthHeader();
    apiRequestArgs.extraHeaders = { ...extraHeaders, ...authHeaders };
    return coreRequest(apiRequestArgs);
}
export async function apiRequest(apiRequestArgs, getAuthHeader) {
    const url = `/api${apiRequestArgs.subpath}`;
    const requestArgs = { ...apiRequestArgs, url, subpath: undefined };
    return rawRequest(requestArgs, getAuthHeader);
}
