import { ERR } from "../util/logging.js";
const defaultClientError = {
    msg: "An internal error occurred! Please check the logs!",
    code: 500,
};
export function clientErrorRoute(errorLoggers, updateClient = true) {
    let defaultHandler = () => { };
    let expectedHandler = defaultHandler;
    let unexpectedHandler = (e, { req }) => ERR("INTERNAL ERROR", `Location: ${req.path}\n`, e);
    if (typeof errorLoggers?.expectedError === "function")
        expectedHandler = errorLoggers.expectedError;
    if (typeof errorLoggers?.unexpectedError === "function")
        unexpectedHandler = errorLoggers.unexpectedError;
    if (errorLoggers?.unexpectedError === false)
        unexpectedHandler = defaultHandler;
    return function clientErrorRoute(e, req, res, _next) {
        const isExpectedError = e instanceof ClientError;
        const clientError = isExpectedError ? e.serialize() : defaultClientError;
        clientError.loc = clientError.loc ?? req.path;
        if (isExpectedError)
            expectedHandler(e, { req, res, next: _next });
        else if (!isExpectedError)
            unexpectedHandler(e, { req, res, next: _next });
        if (res.headersSent)
            ERR("ROUTING", `Client already recieved response! Please check your route and middleware! ${req.path}`);
        if (updateClient)
            res.status(clientError.code).json(clientError);
    };
}
export default class ClientError extends Error {
    statusCode;
    message;
    isError(e) {
        return this.statusCode === e.statusCode && this.message === e.message;
    }
    isSerailized(e) {
        return this.statusCode === e.code && this.message === e.msg;
    }
    serialize() {
        return {
            msg: this.message,
            code: this.statusCode,
            loc: this.location,
            detail: this.detail,
            data: this.data,
        };
    }
    constructor(statusCode, message, optionalArgs) {
        super(message);
        this.statusCode = statusCode;
        this.message = message;
        this.name = "ClientError";
        this.statusCode = statusCode;
        this.message = message;
        this.detail = optionalArgs?.detail;
        this.location = optionalArgs?.location;
        this.data = optionalArgs?.data;
    }
}
