import { Request, Response, NextFunction, RequestHandler } from "express";
import { type InferType, type Schema } from "yup";
import type { OptionsJson } from "body-parser";
import type { IncomingHttpHeaders } from "http";
interface RouteTerms {
    RequestBodyContract?: Schema;
    RequestHeadersContract?: Schema;
    RequestParamsContract?: Schema;
    RequestQueryContract?: Schema;
}
interface RouteResponseTerms extends RouteTerms {
    ResponseBodyContract?: Schema;
}
export interface ContractRouteTerms<T extends RouteResponseTerms = RouteResponseTerms> {
    rsb?: T["ResponseBodyContract"];
    rqb?: T["RequestBodyContract"];
    rqh?: T["RequestHeadersContract"];
    rqp?: T["RequestParamsContract"];
    rqq?: T["RequestQueryContract"];
    resBody?: T["ResponseBodyContract"];
    reqBody?: T["RequestBodyContract"];
    reqHeaders?: T["RequestHeadersContract"];
    reqParams?: T["RequestParamsContract"];
    reqQuery?: T["RequestQueryContract"];
    json?: OptionsJson;
}
export interface ContractRouteContext<T extends RouteTerms = RouteTerms> {
    reqBody: T["RequestBodyContract"] extends Schema ? InferType<T["RequestBodyContract"]> : any;
    reqHeaders: T["RequestHeadersContract"] extends Schema ? InferType<T["RequestHeadersContract"]> : Record<string, string | undefined>;
    reqParams: T["RequestParamsContract"] extends Schema ? InferType<T["RequestParamsContract"]> & IncomingHttpHeaders : IncomingHttpHeaders;
    reqQuery: T["RequestQueryContract"] extends Schema ? InferType<T["RequestQueryContract"]> : Record<string, string | undefined>;
    req: Request;
    res: Response;
    next?: NextFunction;
}
declare type ContractRoute<T extends RouteResponseTerms = RouteResponseTerms> = (routeData: ContractRouteContext<T>) => Promise<InferType<T["ResponseBodyContract"] | any>> | InferType<T["ResponseBodyContract"] | any>;
export interface ContractRouteMetadata {
    reqBodyContract?: Schema;
    resBodyContract?: Schema;
    reqHeadersContract?: Schema;
    reqParamsContract?: Schema;
    reqQueryContract?: Schema;
}
export default function contract<T extends RouteResponseTerms = RouteResponseTerms>(route: ContractRoute<T>, options?: ContractRouteTerms<T>): RequestHandler;
export {};
