import { Router, json as jsonMiddleware } from "express";
import { ValidationError } from "yup";
import ClientError from "../ClientErrors.js";
export default function contract(route, options = {}) {
    const reqBodyContract = options.rqb ?? options.reqBody;
    const resBodyContract = options.rsb ?? options.resBody;
    const reqHeadersContract = options.rqh ?? options.reqHeaders;
    const reqParamsContract = options.rqp ?? options.reqParams;
    const reqQueryContract = options.rqq ?? options.reqQuery;
    const middlewares = Router({ mergeParams: true });
    async function contractMiddleware(req, res, next) {
        const routeObjects = {
            reqBody: req.body,
            reqHeaders: req.headers,
            reqParams: req.params,
            reqQuery: req.query,
            req,
            res,
            next,
        };
        try {
            if (!!reqBodyContract)
                await reqBodyContract.validate(req.body);
            if (!!reqHeadersContract)
                await reqHeadersContract.validate(req.headers);
            if (!!reqParamsContract)
                await reqParamsContract.validate(req.params);
            if (!!reqQueryContract)
                await reqQueryContract.validate(req.query);
        }
        catch (error) {
            if (!(ValidationError.isError(error)))
                throw error;
            throw new ClientError(400, "Bad Request", { detail: "Your request had some errors!", data: error.errors });
        }
        const returnBody = await route(routeObjects);
        if (!!resBodyContract)
            await resBodyContract.validate(returnBody);
        if (!!returnBody)
            return res.json(returnBody);
    }
    middlewares.use([jsonMiddleware(options.json), contractMiddleware]);
    middlewares.routeMetadata = {
        vixContracts: {
            reqBodyContract,
            resBodyContract,
            reqHeadersContract,
            reqParamsContract,
            reqQueryContract,
        },
    };
    return middlewares;
}
