import { Router } from "express";
import ClientError from "../ClientErrors.js";
const middlewares = Router();
export default function tokenAuthMiddleware(authMethod, clientError, middlewareMetadata) {
    async function tokenAuthMiddleware(req, res, next) {
        const { token } = req;
        if (!token)
            throw new Error("Token required and should be present at this layer!");
        const authorized = await authMethod(token);
        const clientErrorType = typeof clientError;
        if (!authorized && !clientError)
            return res.sendStatus(401);
        if (!authorized && clientErrorType === "string")
            return res.status(401).send(clientError);
        if (!authorized && typeof clientErrorType === "object")
            return res.status(401).json(clientError);
        if (!authorized && clientError instanceof ClientError)
            throw clientError;
        if (!next)
            return res.sendStatus(200);
        next();
    }
    middlewares.use(tokenAuthMiddleware);
    middlewares.routeMetadata = middlewareMetadata ?? { useTokenAuth: true };
    return middlewares;
}
