import { Express, Router } from "express";
export interface MetadataRouter extends Router {
    routeMetadata?: Record<string, any>;
}
export interface MappedRoute {
    path: string;
    pathParams: Parameter[];
    method: string;
    routeMetadata?: Record<string, any>;
}
export interface Parameter {
    in: string;
    name: string;
    required: boolean;
    [key: string]: any;
}
export interface Key {
    name: string;
    optional: boolean;
    offset: number;
}
/**
 * Parses an Express app and generates list of routes with meta data
 *
 * @param app The Express app reference. Must be used after all routes have been attached
 * @returns List of routes for this express app with meta-data that has been picked up
 */
export declare const parseExpressApp: (app: Express) => MappedRoute[];
export declare function routePreviews(app: Express, cb?: (r: MappedRoute, methodDisplay: string) => string): string[];
export declare const onlyForTesting: {
    pathRegexParser: (layerRegexPath: RegExp | string, keys: Key[]) => string;
    mapKeysToPath: (layerRegexPath: RegExp, keys: Key[]) => string;
};
